{ DEFINE Debug}
{$A+,B-,F+,G+,E-,N-,P+,T-,V-,X+}
{$IFDEF Debug}
  {$D+,L+,Y+,I+,Q-,R+,S+}
{$ELSE}
  {$D-,L-,Y-,I-,Q-,R-,S-}
{$ENDIF}

UNIT GDates;


INTERFACE


USES
  Dos;


CONST
  MonthsBG: ARRAY[0..12] OF STRING[3] =
    ('---',
     '','','','','','','','','','','','');


TYPE
  TDate = LongInt;
  TTime = LongInt;

  TDateTime = RECORD
      D : TDate;
      T : TTime;
  END;


CONST
  MinYear  = 1600;
  MaxYear  = 3999;
  MinDate  = $00000000;    {= 01/01/1600}
  MaxDate  = $000D6025;    {= 12/31/3999}
  Date1900 = $0001AC05;    {= 01/01/1900}
  Date1980 = $00021E28;    {= 01/01/1980}
  BadDate  = $FFFFFFFF;

  MinTime  = 0;            {= 00:00:00 am}
  MaxTime  = 86399;        {= 23:59:59 pm}
  BadTime  = $FFFFFFFF;

  SecondsInDay    = 86400; {number of seconds in a day}
  SecondsInHour   = 3600;  {number of seconds in an hour}
  SecondsInMinute = 60;    {number of seconds in a minute}
  HoursInDay      = 24;    {number of hours in a day}
  MinutesInHour   = 60;    {number of minutes in an hour}



FUNCTION  Today                                  : TDate;
FUNCTION  IsLeapYear   (Year: Integer)           : Boolean;
FUNCTION  DaysInMonth  (Month, Year: Integer)    : Integer;
FUNCTION  ValidDate    (Day,Month,Year: Integer) : Boolean;
{ Return: Sunday=0, Mon=1, etc... Sat=6; }
FUNCTION  DayOfWeek    (Day,Month,Year: Integer) : Integer;

PROCEDURE TimeToHMS    (T: TTime; VAR Hours,Minutes,Seconds: Byte);
PROCEDURE DateToDMY    (Julian: TDate; VAR Day,Month,Year: Integer);

FUNCTION  HMStoTime    (Hours,Minutes,Seconds: Byte) : TTime;
FUNCTION  DMYtoDate    (Day,Month,Year: Integer)     : TDate;

PROCEDURE TimeDIFF     (Time1,Time2: TTime; VAR Hours,Minutes,Seconds: Byte);
PROCEDURE DateDIFF     (Date1,Date2: TDate; VAR Days,Months,Years: Integer);
PROCEDURE DateTimeDIFF (DT1,DT2: TDateTime; VAR Days:LongInt; VAR Secs: LongInt);
FUNCTION  HoursDIFF    (DT1,DT2: TDateTime): LongInt;

FUNCTION  IncTime      (T:TTime; Hours,Minutes,Seconds: Byte): TTime;
FUNCTION  IncDate      (Julian: TDate; Days,Months,Years: Integer): TDate;
PROCEDURE IncDateTime  (VAR Start,DT1:TDateTime; Days:Integer; Secs:LongInt);

FUNCTION  CurrentTime: TTime;
PROCEDURE GetCurrentDateTime(VAR DateTime: TDateTime);

FUNCTION GetCurrentYear  : Integer;
FUNCTION GetCurrentMonth : Integer;
FUNCTION GetCurrentDay   : Integer;

FUNCTION GetCurrentHour  : Byte;
FUNCTION GetCurrentMin   : Byte;
FUNCTION GetCurrentSec   : Byte;

FUNCTION GetYear         (D:TDate):Integer;
FUNCTION GetMonth        (D:TDate):Integer;
FUNCTION GetDay          (D:TDate):Integer;

FUNCTION GetHour         (T:TTime):Byte;
FUNCTION GetMinute       (T:TTime):Byte;
FUNCTION GetSecond       (T:TTime):Byte;

FUNCTION GetStrDate      (DT:TDateTime) : STRING;
FUNCTION GetStrTime      (DT:TDateTime) : STRING;
FUNCTION GetStrDateTime  (DT:TDateTime) : STRING;

FUNCTION GetCurrentStrDate     : STRING;
FUNCTION GetCurrentStrTime     : STRING;
FUNCTION GetCurrentStrDateTime : STRING;


IMPLEMENTATION


CONST
  First2Months = 59;  {1600 was a leap year}
  FirstDayOfWeek = 6; {01/01/1600 was a Saturday}


FUNCTION IsLeapYear(Year : Integer) : Boolean;
BEGIN
  IsLeapYear := (Year MOD 4 = 0) AND (Year MOD 4000 <> 0) AND
    ((Year MOD 100 <> 0) OR (Year MOD 400 = 0));
END;


FUNCTION DaysInMonth(Month, Year : Integer) : Integer;
BEGIN
  IF Word(Year) < 100 THEN
    Inc(Year, 1900);
  CASE Month OF
    1, 3, 5, 7, 8, 10, 12 :
      DaysInMonth := 31;
    4, 6, 9, 11 :
      DaysInMonth := 30;
    2 :
      DaysInMonth := 28+Ord(IsLeapYear(Year));
  ELSE
    DaysInMonth := 0;
  END;
END;


FUNCTION ValidDate(Day,Month,Year: Integer): Boolean;
BEGIN
  IF Word(Year) < 100 THEN Inc(Year, 1900);

  IF (Day < 1) OR (Year < MinYear) OR (Year > MaxYear) THEN
    ValidDate := False
  ELSE case Month of
    1..12 :
      ValidDate := Day <= DaysInMonth(Month, Year);
  ELSE
    ValidDate := False;
  END
END;


{ Return: Sunday=0, Mon=1, etc... Sat=6; }
FUNCTION DayOfWeek(Day,Month,Year:Integer): Integer;
VAR Century,Holder: Integer;
BEGIN
  { First test For error conditions on input values: }
  IF (Year < 0) OR (Month < 1) OR (Month > 12) OR (Day < 1) OR (Day > 31) THEN
    DayOfWeek := -1  { Return -1 to indicate an error }
  ELSE
  { Do the Zeller's Congruence calculation as Zeller himself }
  { described it in "Acta Mathematica" #7, Stockhold, 1887.  }
  BEGIN
    { First we separate out the year and the century figures: }
    Century := Year DIV 100;
    Year    := Year MOD 100;
    { Next we adjust the month such that March remains month #3, }
    { but that January and February are months #13 and #14,     }
    { *but of the previous year*: }
    IF Month < 3 THEN
    BEGIN
      Inc(Month, 12);
      IF Year > 0 THEN
        Dec(Year, 1)      { The year before 2000 is }
      ELSE                { 1999, not 20-1...       }
      BEGIN
        Year := 99;
        Dec(Century);
      END;
    END;

    { Here's Zeller's seminal black magic: }
    Holder := Day;                        { Start With the day of month }
    Holder := Holder + (((Month + 1) * 26) div 10); { Calc the increment }
    Holder := Holder + Year;              { Add in the year }
    Holder := Holder + (Year div 4);      { Correct For leap years  }
    Holder := Holder + (Century div 4);   { Correct For century years }
    Holder := Holder - Century - Century; { DON'T KNOW WHY HE DID THIS! }
    {***********************KLUDGE ALERT!***************************}
    WHILE Holder < 0 DO                   { Get negative values up into }
      Inc(Holder, 7);                     { positive territory before   }
                                          { taking the MOD...         }
    Holder := Holder MOD 7;               { Divide by 7 but keep the  }
                                          { remainder rather than the }
                                          { quotient }
    {***********************KLUDGE ALERT!***************************}
    { Here we "wrap" Saturday around to be the last day: }
    IF Holder = 0 THEN
      Holder := 7;

    { Zeller kept the Sunday = 1 origin; computer weenies prefer to }
    { start everything With 0, so here's a 20th century kludge:     }
    Dec(Holder);

    DayOfWeek := Holder;  { Return the end product! }
  END;
END;


FUNCTION DMYtoDate(Day,Month,Year: Integer): TDate;
BEGIN
  IF Word(Year) < 100 THEN Inc(Year, 1900);

  IF NOT ValidDate(Day, Month, Year) THEN
    DMYtoDate := BadDate
  ELSE IF (Year = MinYear) and (Month < 3) THEN
    IF Month = 1 THEN
      DMYtoDate := Pred(Day)
    ELSE
      DMYtoDate := Day+30
  ELSE BEGIN
    IF Month > 2 THEN
      Dec(Month, 3)
    ELSE BEGIN
      Inc(Month, 9);
      Dec(Year);
    END;
    Dec(Year, MinYear);
    DMYtoDate :=
      ((LongInt(Year div 100)*146097) div 4)+
      ((LongInt(Year mod 100)*1461) div 4)+
      (((153*Month)+2) div 5)+Day+First2Months;
  END;
END;


PROCEDURE DateToDMY(Julian: TDate; VAR Day,Month,Year: Integer);
VAR I,J:LongInt;
BEGIN
  IF Julian = BadDate THEN BEGIN
    Day := 0;
    Month := 0;
    Year := 0;
  END
  ELSE IF Julian <= First2Months THEN BEGIN
    Year := MinYear;
    IF Julian <= 30 THEN BEGIN
      Month := 1;
      Day := Succ(Julian);
    END
    ELSE BEGIN
      Month := 2;
      Day := Julian-30;
    END;
  END
  ELSE BEGIN
    I := (4*LongInt(Julian-First2Months))-1;
    J := (4*((I MOD 146097) DIV 4))+3;
    Year := (100*(I div 146097))+(J div 1461);
    I := (5*(((J mod 1461)+4) div 4))-3;
    Month := I div 153;
    Day := ((I mod 153)+5) div 5;
    IF Month < 10 THEN
      Inc(Month, 3)
    ELSE BEGIN
      Dec(Month, 9);
      Inc(Year);
    END;
    Inc(Year, MinYear);
  END;
END;


FUNCTION IncDate(Julian: TDate; Days,Months,Years: Integer): TDate;
VAR
  Day, Month, Year, Day28Delta : Integer;
BEGIN
  DateToDMY(Julian, Day, Month, Year);
  Day28Delta := Day-28;
  IF Day28Delta < 0 THEN
    Day28Delta := 0
  ELSE
    Day := 28;
  Inc(Month, Months-1);
  Inc(Year, Years+(Month div 12)-Ord(Month < 0));
  Month := Succ(Month mod 12);
  IF Month <= 0 THEN
    Inc(Month, 12);
  Julian := DMYtoDate(Day, Month, Year);
  IF Julian <> BadDate THEN BEGIN
    Inc(Julian, Days);
    Inc(Julian, Day28Delta);
  END;
  IncDate := Julian;
END;


PROCEDURE DateDIFF(Date1,Date2: TDate; VAR Days,Months,Years: Integer);
VAR
  Day1, Day2, Month1, Month2, Year1, Year2 : Integer; X : LongInt;
BEGIN
  {we want Date2 > Date1}
  IF Date1 > Date2 THEN
  BEGIN
    X     := Date1;
    Date1 := Date2;
    Date2 := X;
  END;

  {convert dates to day,month,year}
  DateToDMY(Date1, Day1, Month1, Year1);
  DateToDMY(Date2, Day2, Month2, Year2);

  {days first}
  IF Day2 < Day1 THEN BEGIN
    Dec(Month2);
    IF Month2 = 0 THEN BEGIN
      Month2 := 12;
      Dec(Year2);
    END;
    Inc(Day2, DaysInMonth(Month2, Year2));
  END;
  Days := Day2-Day1;

  {now months and years}
  IF Month2 < Month1 THEN BEGIN
    Inc(Month2, 12);
    Dec(Year2);
  END;
  Months := Month2-Month1;
  Years := Year2-Year1;
END;


FUNCTION Today: TDate;
VAR
  Year, Month, Day, DayOfWeek : Word;
BEGIN
  GetDate(Year, Month, Day, DayOfWeek);
  Today := DMYtoDate(Day, Month, Year);
END;







PROCEDURE TimeToHMS(T: TTime; VAR Hours,Minutes,Seconds: Byte);
BEGIN
  IF T = BadTime THEN BEGIN
    Hours := 0;
    Minutes := 0;
    Seconds := 0;
  END
  ELSE BEGIN
    Hours := T div SecondsInHour;
    Dec(T, LongInt(Hours)*SecondsInHour);
    Minutes := T div SecondsInMinute;
    Dec(T, LongInt(Minutes)*SecondsInMinute);
    Seconds := T;
  END;
END;


FUNCTION HMStoTime(Hours, Minutes, Seconds : Byte): TTime;
VAR
  T: TTime;
BEGIN
  Hours := Hours mod HoursInDay;
  T := (LongInt(Hours)*SecondsInHour)+(LongInt(Minutes)*SecondsInMinute)+Seconds;
  HMStoTime := T mod SecondsInDay;
END;



PROCEDURE TimeDIFf(Time1,Time2: TTime; VAR Hours,Minutes,Seconds: Byte);
VAR
  T: TTime;
BEGIN
  IF Time1 > Time2 THEN
    T := Time1-Time2
  ELSE
    T := Time2-Time1;
  TimeToHMS(T, Hours, Minutes, Seconds);
END;


FUNCTION IncTime(T: TTime; Hours,Minutes,Seconds: Byte): TTime;
BEGIN
  Hours := Hours mod HoursInDay;
  Inc(T, HMStoTime(Hours, Minutes, Seconds));
  IncTime := T mod SecondsInDay;
END;


FUNCTION CurrentTime: TTime;
VAR
  Hours, Minutes, Seconds, Sec100 : Word;
BEGIN
  GetTime(Hours, Minutes, Seconds, Sec100);
  CurrentTime := HMStoTime(Hours, Minutes, Seconds);
END;






PROCEDURE DateTimeDIFf(DT1,DT2: TDateTime; VAR Days:LongInt; VAR Secs: LongInt);
VAR X:TDateTime;
BEGIN
  {swap IF DT1 later than DT2}
  IF (DT1.D > DT2.D) or ((DT1.D = DT2.D) and (DT1.T > DT2.T)) THEN
  BEGIN
    X   := DT1;
    DT1 := DT2;
    DT2 := X;
  END;

  {the dIFference in days is easy}
  Days := DT2.D-DT1.D;

  {dIFference in seconds}
  IF DT2.T < DT1.T THEN BEGIN
    {subtract one day, add 24 hours}
    Dec(Days);
    Inc(DT2.T, SecondsInDay);
  END;
  Secs := DT2.T-DT1.T;
END;



PROCEDURE IncDateTime(VAR Start,DT1:TDateTime; Days:Integer; Secs:LongInt);
BEGIN
  DT1 := Start;

  {date first}
  Inc(DT1.D, LongInt(Days));

  IF Secs < 0 THEN BEGIN
    {change the sign}
    Secs := -Secs;

    {adjust the date}
    Dec(DT1.D, Secs DIV SecondsInDay);
    Secs := Secs MOD SecondsInDay;

    IF Secs > DT1.T THEN BEGIN
      {subtract a day from DT2.D and add a day's worth of seconds to DT2.T}
      Dec(DT1.D);
      Inc(DT1.T, SecondsInDay);
    END;

    {now subtract the seconds}
    Dec(DT1.T, Secs);
  END
  ELSE BEGIN
    {increment the seconds}
    Inc(DT1.T, Secs);

    {adjust date IF necessary}
    Inc(DT1.D, DT1.T DIV SecondsInDay);

    {force time to 0..SecondsInDay-1 range}
    DT1.T := DT1.T MOD SecondsInDay;
  END;
END;



FUNCTION HoursDIFF(DT1,DT2: TDateTime): LongInt;
VAR Days:LongInt; Secs: LongInt; A,B:LongInt;
BEGIN
  DateTimeDIFF(DT1,DT2,Days,Secs);
  HoursDIFF := (Days * HoursInDay) + (Secs DIV SecondsInHour);
END;



PROCEDURE GetCurrentDateTime(VAR DateTime: TDateTime);
VAR
  Year, Month, Day, DayOfWeek : Word;
  Hours, Minutes, Seconds, Sec100 : Word;
BEGIN
  GetDate(Year, Month, Day, DayOfWeek);
  GetTime(Hours, Minutes, Seconds, Sec100);
  DateTime.D := DMYtoDate(Day, Month, Year);
  DateTime.T := HMStoTime(Hours, Minutes, Seconds);
END;



FUNCTION GetYear(D:TDate):Integer;
VAR Day,Month,Year:Integer;
BEGIN
  DateToDMY(D,Day,Month,Year);
  GetYear := Year;
END;

FUNCTION GetMonth(D:TDate):Integer;
VAR Day,Month,Year:Integer;
BEGIN
  DateToDMY(D,Day,Month,Year);
  GetMonth := Month;
END;

FUNCTION GetDay(D:TDate):Integer;
VAR Day,Month,Year:Integer;
BEGIN
  DateToDMY(D,Day,Month,Year);
  GetDay := Day;
END;


FUNCTION GetHour(T:TTime):Byte;
VAR Hour,Min,Sec:Byte;
BEGIN
  TimeToHMS(T,Hour,Min,Sec);
  GetHour := Hour;
END;

FUNCTION GetMinute(T:TTime):Byte;
VAR Hour,Min,Sec:Byte;
BEGIN
  TimeToHMS(T,Hour,Min,Sec);
  GetMinute := Min;
END;

FUNCTION GetSecond(T:TTime):Byte;
VAR Hour,Min,Sec:Byte;
BEGIN
  TimeToHMS(T,Hour,Min,Sec);
  GetSecond := Sec;
END;




FUNCTION GetCurrentYear: Integer;
VAR Year,Month,Day,DayOfWeek: Word;
BEGIN
  GetDate(Year,Month,Day,DayOfWeek);
  GetCurrentYear := Year;
END;


FUNCTION GetCurrentMonth: Integer;
VAR Year,Month,Day,DayOfWeek: Word;
BEGIN
  GetDate(Year,Month,Day,DayOfWeek);
  GetCurrentMonth := Month;
END;


FUNCTION GetCurrentDay: Integer;
VAR Year,Month,Day,DayOfWeek: Word;
BEGIN
  GetDate(Year,Month,Day,DayOfWeek);
  GetCurrentDay := Day;
END;


FUNCTION GetCurrentHour: Byte;
VAR Hour,Minute,Second,Sec100: Word;
BEGIN
  GetTime(Hour,Minute,Second,Sec100);
  GetCurrentHour := Hour;
END;


FUNCTION GetCurrentMin: Byte;
VAR Hour,Minute,Second,Sec100: Word;
BEGIN
  GetTime(Hour,Minute,Second,Sec100);
  GetCurrentMin := Minute;
END;


FUNCTION GetCurrentSec: Byte;
VAR Hour,Minute,Second,Sec100: Word;
BEGIN
  GetTime(Hour,Minute,Second,Sec100);
  GetCurrentSec := Second;
END;






FUNCTION GetStrDate(DT:TDateTime):STRING;
VAR Day,Month,Year: Integer; sDay:STRING[2]; sYear:STRING[4];
BEGIN
  DateToDMY(DT.D,Day,Month,Year);
  Str(Year,sYear);
  Str(Day,sDay);
  IF Day < 10 THEN sDay := '0'+sDay;
  GetStrDate := sDay+'-'+MonthsBG[Month]+'-'+sYear;
END;

FUNCTION GetStrTime(DT:TDateTime):STRING;
VAR Hour,Min,Sec:Byte; sHour,sMin,sSec:STRING[2];
BEGIN
  TimeToHMS(DT.T,Hour,Min,Sec);
  Str(Hour,sHour);
  Str(Min ,sMin);
  Str(Sec ,sSec);
  IF Hour < 10 THEN sHour := '0'+sHour;
  IF Min  < 10 THEN sMin  := '0'+sMin;
  IF Sec  < 10 THEN sSec  := '0'+sSec;
  GetStrTime := sHour+':'+sMin+':'+sSec;
END;


FUNCTION GetStrDateTime(DT:TDateTime):STRING;
BEGIN
  GetStrDateTime := GetStrDate(DT) + ' ' + GetStrTime(DT);
END;


FUNCTION GetCurrentStrDate:STRING;
VAR DateTime:TDateTime;
BEGIN
  GetCurrentDateTime(DateTime);
  GetCurrentStrDate := GetStrDate( DateTime );
END;


FUNCTION GetCurrentStrTime:STRING;
VAR DateTime:TDateTime;
BEGIN
  GetCurrentDateTime(DateTime);
  GetCurrentStrTime := GetStrTime( DateTime );
END;


FUNCTION GetCurrentStrDateTime:STRING;
VAR DateTime:TDateTime;
BEGIN
  GetCurrentDateTime(DateTime);
  GetCurrentStrDateTime := GetStrDateTime( DateTime );
END;



END.
